; inkscape GtkAccelMap rc-file         -*- scheme -*-
; Simply edit the key bindings you wish to override and restart Inkscape.
;
; File menu
;TODO: VacuumDefs
(gtk_accel_path "<Actions>//New" "<Control>n")
(gtk_accel_path "<Actions>//Open" "<Control>o")
(gtk_accel_path "<Actions>//Revert" "<Shift><Control>r")
(gtk_accel_path "<Actions>//Save" "<Control>s")
(gtk_accel_path "<Actions>//SaveAs" "<Shift><Control>s")
(gtk_accel_path "<Actions>//Import" "<Control>i")
(gtk_accel_path "<Actions>//Export" "<Shift><Control>e")
(gtk_accel_path "<Actions>//Print" "<Control>p")
(gtk_accel_path "<Actions>//PrintPreview" "<Shift><Control>p")
(gtk_accel_path "<Actions>//DocumentProperties" "<Shift><Control>d")
(gtk_accel_path "<Actions>//InkscapePreferences" "<Shift><Control>p")
(gtk_accel_path "<Actions>//Close" "<Control>w")
(gtk_accel_path "<Actions>//Quit" "<Control>q")
;
; Edit menu
(gtk_accel_path "<Actions>//Undo" "<Control>z")
(gtk_accel_path "<Actions>//Redo" "<Control>y")
(gtk_accel_path "<Actions>//Cut" "<Control>x")
(gtk_accel_path "<Actions>//Copy" "<Control>c")
(gtk_accel_path "<Actions>//Paste" "<Control>v")
(gtk_accel_path "<Actions>//PasteInPlace" "<Control><Alt>v")
(gtk_accel_path "<Actions>//PasteStyle" "<Shift><Control>v")
(gtk_accel_path "<Actions>//Find" "<Control>f")
(gtk_accel_path "<Actions>//Duplicate" "<Control>d")
(gtk_accel_path "<Actions>//Clone" "<Alt>d")
(gtk_accel_path "<Actions>//CloneUnlink" "<Shift><Alt>d")
(gtk_accel_path "<Actions>//CloneSelectOrig" "<Shift>d")
(gtk_accel_path "<Actions>//MakeBitmap" "<Alt>b")
(gtk_accel_path "<Actions>//Tile" "<Alt>i")
(gtk_accel_path "<Actions>//Untile" "<Shift><Alt>i")
(gtk_accel_path "<Actions>//Delete" "Delete")
(gtk_accel_path "<Actions>//SelectAll" "<Control>a")
(gtk_accel_path "<Actions>//SelectNone" "<Shift><Control>a")
(gtk_accel_path "<Actions>//XmlEditor" "<Shift><Control>x")
;
; View menu
;TODO: ShowHide Messages WindowDuplicate
(gtk_accel_path "<Actions>//ZoomIn" "plus")
(gtk_accel_path "<Actions>//ZoomOut" "minus")
(gtk_accel_path "<Actions>//Zoom100" "1")
(gtk_accel_path "<Actions>//Zoom50" "2")
(gtk_accel_path "<Actions>//Zoom200" "")
(gtk_accel_path "<Actions>//ZoomSelection" "3")
(gtk_accel_path "<Actions>//ZoomDrawing" "4")
(gtk_accel_path "<Actions>//ZoomPage" "5")
(gtk_accel_path "<Actions>//ZoomWidth" "6")
(gtk_accel_path "<Actions>//ZoomPrevious" "grave")
(gtk_accel_path "<Actions>//ZoomNext" "<Shift>grave")
(gtk_accel_path "<Actions>//ShowHideDialogs" "F12")
(gtk_accel_path "<Actions>//Messages" "")
(gtk_accel_path "<Actions>//Grid" "numbersign")
(gtk_accel_path "<Actions>//Guides" "bar")
(gtk_accel_path "<Actions>//Fullscreen" "F11")
(gtk_accel_path "<Actions>//WindowPrevious" "<Shift><Control>Tab")
(gtk_accel_path "<Actions>//WindowNext" "<Control>Tab")
;
; Layer menu
;TODO: NewLayer RenameLayer SwitchToNextLayer SwitchToPrevLayer DeleteCurrentLayer
(gtk_accel_path "<Actions>//RaiseLayer" "<Shift><Control>Page_Up")
(gtk_accel_path "<Actions>//LowerLayer" "<Shift><Control>Page_Down")
(gtk_accel_path "<Actions>//LayerToTop" "<Shift><Control>Home")
(gtk_accel_path "<Actions>//LayerToBottom" "<Shift><Control>End")
;
; Object menu
;TODO: MoveToNewLayer MoveToNextLayer MoveToPrevLayer MoveToTopLayer MoveToBottomLayer
(gtk_accel_path "<Actions>//FillAndStroke" "<Shift><Control>f")
(gtk_accel_path "<Actions>//ObjectProperties" "<Shift><Control>o")
(gtk_accel_path "<Actions>//Group" "<Control>g")
(gtk_accel_path "<Actions>//Ungroup" "<Control>u")
(gtk_accel_path "<Actions>//Raise" "Page_Up")
(gtk_accel_path "<Actions>//Lower" "Page_Down")
(gtk_accel_path "<Actions>//RaiseToTop" "Home")
(gtk_accel_path "<Actions>//LowerToBottom" "End")
(gtk_accel_path "<Actions>//Rotate90CW" "<Shift><Control>Right")
(gtk_accel_path "<Actions>//Rotate90CCW" "<Shift><Control>Left")
(gtk_accel_path "<Actions>//FlipHoriz" "h")
(gtk_accel_path "<Actions>//FlipVert" "v")
(gtk_accel_path "<Actions>//Transform" "<Shift><Control>m")
(gtk_accel_path "<Actions>//AlignAndDistribute" "<Shift><Control>a")
;
; Path menu
;TODO: Reverse Cleanup
(gtk_accel_path "<Actions>//ObjectToPath" "<Shift><Control>c")
(gtk_accel_path "<Actions>//StrokeToPath" "<Control><Alt>c")
(gtk_accel_path "<Actions>//Trace" "<Shift><Alt>b")
(gtk_accel_path "<Actions>//Union" "<Control>plus")
(gtk_accel_path "<Actions>//Difference" "<Control>minus")
(gtk_accel_path "<Actions>//Intersection" "<Control>asterisk")
(gtk_accel_path "<Actions>//Exclusion" "<Control>asciicircum")
(gtk_accel_path "<Actions>//Division" "<Control>slash")
(gtk_accel_path "<Actions>//CutPath" "<Control><Alt>slash")
(gtk_accel_path "<Actions>//Combine" "<Control>k")
(gtk_accel_path "<Actions>//BreakApart" "<Shift><Control>k")
(gtk_accel_path "<Actions>//Inset" "<Control>parenleft")
(gtk_accel_path "<Actions>//Outset" "<Control>parenright")
(gtk_accel_path "<Actions>//OffsetDynamic" "<Control>j")
(gtk_accel_path "<Actions>//OffsetLinked" "<Control><Alt>j")
(gtk_accel_path "<Actions>//Simplify" "<Control>l")
;
; Text menu
;TODO: PutOnPath RemoveFromPath RemoveManualKerns
(gtk_accel_path "<Actions>//TextProperties" "<Shift><Control>t")
;
; About menu
;TODO: KeysAndMouse Tutorials About
;
; Tools toolbox
(gtk_accel_path "<Actions>//ToolSelect" "F1")
(gtk_accel_path "<Actions>//ToolNode" "F2")
(gtk_accel_path "<Actions>//ToolZoom" "F3")
(gtk_accel_path "<Actions>//ToolRect" "F4")
(gtk_accel_path "<Actions>//ToolArc" "F5")
(gtk_accel_path "<Actions>//ToolStar" "asterisk")
(gtk_accel_path "<Actions>//ToolSpiral" "F9")
(gtk_accel_path "<Actions>//ToolFreehand" "F6")
(gtk_accel_path "<Actions>//ToolPen" "<Shift>F6")
(gtk_accel_path "<Actions>//ToolDynaDraw" "<Control>F6")
(gtk_accel_path "<Actions>//ToolText" "F8")
(gtk_accel_path "<Actions>//ToolDropper" "F7")
;
;TODO: tool controls and other actions
